#ifndef __QLIST_H
#define __QLIST_H

#include <vector>
#include "qcollection.h"

template <class T>
class QList : public vector<T*>, public QCollection
{
 private:
  int m_nCurrent;
 public:
  QList() { m_nCurrent = 0;}
  append(const T * t) { push_back((T*)t); }
  int count() const { return size(); }
  bool isEmpty() const { return empty(); }

  T * current() { 
    if(size())
      return (*this)[m_nCurrent];
    else
      return NULL;
  }

  T * prev() { 
    if(m_nCurrent>=0) {
      return (*this)[--m_nCurrent];
    } else {
      return NULL;
    }
  }

  T * next() { 
    if(m_nCurrent<(size()-1))
      return (*this)[++m_nCurrent];
    else
      return NULL;
  }
  
  int at() const {
    if(size())
      return m_nCurrent;
    else
      return -1;
  }

  T * at(const int & i) { 
    if(i<(size())) {
      m_nCurrent = i;
      return (*this)[i];
    } else {
      return NULL;
    }
  }

  bool removeLast() {
    if(size()) {
      pop_back();
      return true;
    } else {
      return false;
    }
  }

  T* take(unsigned int index) {
    if(!size())
      return NULL;

    T* tmp = (*this)[0];
    
    vector<T*>::iterator pos;
    erase(begin());

    if(!size())
      m_nCurrent = -1;

    if(m_nCurrent > size())
      m_nCurrent = size();
    
    return tmp;
  }


  T* getFirst() const {
    if(size())
      return (*this)[0];
    else
      return NULL;
  }

  T* getLast() const {
    if(size())
      return (*this)[size()-1];
    else
      return NULL;
  }

  T* first() {
    if(size()) {
      m_nCurrent = 0;;
      return (*this)[0];
    } else {
      return NULL;
    }
  }

  T* last() { 
    if(size()) {
      m_nCurrent = size()-1;
      return (*this)[size()-1];
    } else {
      return NULL;
    }
  }

  bool remove(const T* item)
    {
      return removeRef(item);
    }

  int findRef(const T* t) 
    { 
      int i;
      for(i=0; i<size(); i++) {
	if((*this)[i] == t) {
	  m_nCurrent = i;	  
	  return i;
	}
      }  
      return -1;
    }

  bool removeRef(const T* item)
    {
      vector<T*>::iterator pos;
      for(pos = begin(); pos != end();) {
	if(*pos == item) {
	  erase(pos);
	  break;
	} else {
	  ++pos;
	}
      }
    }
};

#endif
