#ifndef __QWIDGET_H
#define __QWIDGET_H

#include "fltk-qbase.h"
#include "qpaintdevice.h"
#include "qcursor.h"
#include "qpalette.h"
#include "qsize.h"
#include "qfontmetrics.h"
#include "qfontinfo.h"
#include "Fl_Widget.H"
#include "fl_draw.H"
#include <iostream.h>
#include "qevent.h"

class QWidget : public QObject, public QPaintDevice
{
 protected:

  QCursor m_Cursor;
  QColor m_Color;
  QPalette m_Palette;
  
  QSize m_MinSize;
  QFont m_Font;
  QFontMetrics m_FontMetrics;
 protected:
  virtual void keyPressEvent(QKeyEvent * e) { }

 public:
  QWidget() { }
  QWidget(QWidget * _parent, const char * _name=0, WFlags f=0) { }

  int height() const { return w(); }
  int width() const { return h(); }
  int x() const { return parent()->x() - Fl_Widget::x(); }
  int y() const { return parent()->y() - Fl_Widget::y(); }

  virtual void paintEvent(QPaintEvent * _pe) { cerr << "Virtual paint\n"; }

  virtual FL_EXPORT void draw() 
    { 
      cerr << "::draw " << this << "\n";
      cerr << "::draw " << x() << " " << y() << " " 
	   << w() << " " << h() << "\n";
      QRect r(x(),y(),w(),h());
      QPaintEvent pe(r);
      
      paintEvent(&pe);
      //      fl_color(FL_BLACK);
      //      fl_line(10,10,100,100);  
    }

  virtual int handle(int event)
    {
      return 0;
    }


  bool isVisible() const { return false; }
  //  virtual void hide() { } 
  void raise() { }
  //  virtual void show() { }


  void setMinimumSize(int w, int h) { } 
  void setMinimumSize(const QSize & size) { m_MinSize = size; }
  virtual void setCursor(const QCursor & cursor) { m_Cursor = cursor; }
  virtual void setFocusProxy(QWidget * w) { }
  virtual void setGeometry(int _x, int _y, int _w, int _h)
    {
      cerr << "setGeometry " << _x << " " << _y << " " << _w << " " <<
	_h << "\n";
      Fl_Widget::resize(_x,_y,_w,_h);

      cerr << "setGeometry after " << x() << " " << y() 
	   << " " << w() << " " << h() << "\n";
    }

  virtual void setGeometry(const QRect & rect)
    {
      setGeometry(rect.x(), rect.y(),
		  rect.width(), rect.height());
    }

  QPoint mapToGlobal( const QPoint & point) const { return QPoint(1,1); }
  QPoint mapFromGlobal( const QPoint & point) const { return QPoint(1,1); }
  void repaint() { }
  void repaint(bool erase) { }
  void repaint(int x, int y, int w, int h, bool erase = TRUE) { }
  virtual void setMouseTracking(bool enable) { } 

  virtual void setBackgroundColor(const QColor & color) { m_Color = color; }
  virtual void move(int x, int y) { } 
  void grabMouse() { }
  void releaseMouse() { }
  const QPalette & palette() { return m_Palette; }
  bool isUpdatesEnabled() const { return true; }
  virtual void resize(const QSize & size) 
    { resize(size.width(), size.height()); }
  virtual void resize(int w, int h) { setGeometry(x(),y(),w,h); }
  virtual QSize sizeHint() const { return m_MinSize; }
  void setFont(const QFont & font) { m_Font = font; }
  QFont font() const { return m_Font; }
  QFontMetrics fontMetrics() const { return m_FontMetrics; }
};

#endif
